// v0.9.0
// (c) 2014 Frank Bartels (frank@toppoint.de)
// Licence (GPL3): see LICENCE.txt
//
// pacman() animation is (c) Adrian (same licence)

#include "LightMatrixKernelLib.h"

// MatrixKernel features:  (might not work here!!)
int time_turbo=1; // std=1 more(n)= n times faster
int slowmotion=1; // std=1 more(n)= n times slower
int DO_show_as_ascii=0; // std=0  1=show as ASCII output pseudo graphics

/*
User Interface:
  paint(x,y, r,g,b);     paints a pixel at x,y (outside (0..SCRdx,0..SCRdy) is safe) with color r,g,b
  paint_hi(x,y, r,g,b);  also paints a dot at x,y with color r,g,b, but
                         1. x,y are fixed point numbers (int value FIXP is 1.0)
                         2. r,g,b can grow above 255! Result: Light floods in surrounding pixels

void PrototypeAnimation(int len_s)
{ 
  // declare your variables here, e.g.  int x=0;
  
  int wait=100;                  // std loop (defines 100 ms frames)
  int counter=1000/wait*len_s;   // std loop (takes parameter len_s length in seconds)
  while (counter--)              // std loop
  { frame_delay(wait);           // std loop (waits till last frame_delay endtime plus wait ms)
    swap_buffers(1);             // std loop (double buffering, as expected with starts blank=black)
    
    // make some simulation and output here
    // e.g.:  paint(x,0,  255,0,0);   // red pixel at x,0 (0 is top of screen)
    //        if (++x >= SCRdx) x=0;  // moving right (++), if at border restarts at 0
    
  }                              // std loop end
}
*/

void user_init()  // arduino init() is called in lib, so if you want to init something, do it here
{ 
}

void ForestFire(int len_s)
{ unsigned char forest[SCRdx][SCRdy]; // 0:empty 1:tree 2:fire
  unsigned char cell;
  for(int x=0; x<SCRdx; x++)
  { for(int y=0; y<SCRdy; y++)
    { cell=random(0,2);
      forest[x][y]=cell;
      paint(x,y, cell>1?255:0, cell>0?127:0, 0);
    }
    if (x&&3==0) { frame_delay(50); swap_buffers(2); }
  }

  int wait=100;                  // std loop (defines 100 ms frames)
  int counter=1000/wait*len_s;   // std loop (takes parameter len_s length in seconds)
  while (counter--)              // std loop
  { frame_delay(wait);           // std loop
    swap_buffers(1);             // std loop

    for(int x=0; x<SCRdx; x++)
    { for(int y=0; y<SCRdy; y++)
      { cell=(forest[x][y]&3);
        if (cell==0)
          cell=(random(0,33)==1?1:0);  // grow tree p=3%
        else if (cell==1)
        { int neighbours=0;
          for(int xx=-1; xx<=1; xx++)
            for(int yy=-1; yy<=1; yy++)
              if (!(xx==0 && yy==0) && 
                  x+xx>=0 && x+xx<SCRdx &&
                  y+yy>=0 && y+yy<SCRdy)
                if ((forest[x+xx][y+yy]&3)==2) { neighbours++; goto stop_neighbours; }
          stop_neighbours:
          if (neighbours || random(0,500)==1) cell=2;  // burn f=0.2%
        } 
        else
          cell=0; // stop burning
        forest[x][y]= (forest[x][y]&3)|(cell<<2); // save new value
        paint(x,y, cell>1?255:0, cell>0?127:0, 0);
      }
    }
    for(int x=0; x<SCRdx; x++)
      for(int y=0; y<SCRdy; y++)
        forest[x][y]= forest[x][y]>>2; // new value to old value        
  }
}

void KnightRider()
{
  // Benutzervariablen
  int x=0;
  int dx=1;
  int width=20;
  
  int wait=20;
  int len_s=6;
  int counter=1000/wait*len_s;
  while (counter--)
  { frame_delay(wait);  // Standard-Schleife für Animationen-Kram
    swap_buffers(1); // Standard-Schleife für Animationen-Kram
    
    x=x+dx;
    if (x<=1-width)       { x=1;       dx=1;  }
    if (x>=SCRdx+width-1) { x=SCRdx-2; dx=-1; }
    
    for (int i=0; i<width; i++)
    { int hell=255-(i+0)*255/width;
      for (int y=0; y<SCRdy; y++)
        paint(x-i*dx,y, hell,0,0);
    }      
  }
}

void Sparkles(int len_s)
{ 
  const int STARs = 32; // max.127 (bei genug speicher), realistisch max. ca. 16
  struct star
  { int x,y,dx,dy;
    int r,g,b;
    unsigned char t;
    char nextfree;
  } stars[STARs];
  int firstfree=0;
  int actstars=0;
  for (int i=0; i<STARs; i++)
    stars[i].nextfree= i<STARs-1 ?i+1 :-1;
  int i,n;

  int wait=40;
  int counter=1000/wait*len_s;
  while (counter--)
  { frame_delay(wait);
    swap_buffers(1);

    if (actstars<STARs /*&& random(0,2)==0*/)
    { n=firstfree; // if (n<0) ...bad
        if (n<0) { Serial.println(999999); return; }
      actstars++;
      struct star *e=&stars[n];
      firstfree=e->nextfree;
      e->nextfree=-2;
      e->x=random(3*FIXP,(SCRdx-4)*FIXP);
      e->y=((SCRdy-1)*FIXP);
      e->dx=random(-FIXP/8, FIXP/8);
      e->dy=0*random(-FIXP*2/0x100,-FIXP*1/0x100);
      e->r=random(0xf0,0x100); 
      e->g=random(0x80,0x86);
      e->b=random(0x40,0xa0);
      
      e->t=0;
    }
  
    for (int i=0; i<STARs; i++)
    { struct star *e=&stars[i];
      if (e->nextfree>=-1) continue;
      
      e->x+=e->dx;
      e->y+=e->dy;
      e->dx=e->dx*7/8;
      e->dy-=FIXP/0x100;
      if(0)
      { e->r+=(e->r>>6);
        e->g+=(e->g>>6);
        e->b+=(e->b>>6);
      }
      e->t++;

      paint_hi(e->x,e->y,e->r,e->g,e->b);
      if (e->y>SCRdy_fixp+FIXP || e->y<0 || e->x<0 || e->x>SCRdx_fixp || e->t==148
         || random(0,0x1c)==0)
      { e->nextfree=firstfree;
        firstfree=i;
        actstars--;
      }
    }
  }
}

void Fireworks(int len_s)
{ 
  const int STARs = 16;  // max.127 (bei genug speicher), realistisch max. ca. 16
  struct star
  { int x,y,dx,dy;
    int r,g,b;
    unsigned char t;
    char nextfree;
  } stars[STARs];
  int firstfree=0;
  int actstars=0;
  for (int i=0; i<STARs; i++)
    stars[i].nextfree= i<STARs-1 ?i+1 :-1;
  int i,n;

  int wait=100;
  int counter=1000/wait*len_s;
  while (counter--)
  { frame_delay(wait);
    swap_buffers(1);

    if (actstars<STARs && (counter&0x07)==1)
    { n=firstfree; // if (n<0) ...bad
        if (n<0) { Serial.println(999999); return; }
      actstars++;
      struct star *e=&stars[n];
      firstfree=e->nextfree;
      e->nextfree=-2;
      e->x=SCRdx_fixp/2;
      e->y=SCRdy_fixp;
      e->dx=random((-FIXP)/0x2,(FIXP+1)/0x2);
      e->dy=random(-FIXP*2/3,-FIXP/2);
      e->r=random(0,0x20); 
      e->g=random(0,0x20);
      e->b=0x20 -e->r/2 -e->g/2;
      e->r*=e->r;
      e->g*=e->g;
      e->b*=e->b;
      
      e->t=0;
    }
  
    for (int i=0; i<STARs; i++)
    { struct star *e=&stars[i];
      if (e->nextfree>=-1) continue;
      
      e->x+=e->dx;
      e->y+=e->dy;
      if ((counter&0x1f)==0x1f)
      { e->r-=(e->r>>3);
        e->g-=(e->g>>3);
        e->b-=(e->b>>3);
      }
      e->dy+=(e->t>14 ? FIXP/0x10 : FIXP/0x40);
      e->t++;
      if (e->t==14)
      { for (int ii=0; ii<26 && actstars<STARs; ii++)
        { n=firstfree; // if (n<0) ...bad
            if (n<0) { Serial.println(999999); return; }
          actstars++;
          struct star *ee=&stars[n];
          firstfree=ee->nextfree;
          ee->nextfree=-2;
          ee->x=e->x;
          ee->y=e->y;
          ee->dx=random(-3*FIXP/2,3*FIXP/2+1);
          ee->dy=random(-3*FIXP/2,3*FIXP/2+1);
          ee->r=e->r/2; 
          ee->g=e->g/2;
          ee->b=e->b/2;
          ee->t=e->t+random(1,8);
        }
        e->nextfree=firstfree;
        firstfree=i;
        actstars--;
      }

      paint_hi(e->x,e->y,e->r,e->g,e->b);
      if (e->y>SCRdy_fixp || e->x<0 || e->x>SCRdx_fixp || e->t==28)
      { e->nextfree=firstfree;
        firstfree=i;
        actstars--;
      }
    }
  }
}

void SlimeDrops(int len_s)
{ 
  const int THINGs = 16;  // max.127 (bei genug speicher), realistisch max. ca. 16
  struct thing
  { int x,y,dx,dy;
    int r,g,b;
    int t;
    char nextfree;
  } things[THINGs];
  int firstfree=0;
  int actthings=0;
  for (int i=0; i<THINGs; i++)
    things[i].nextfree= i<THINGs-1 ?i+1 :-1;
  int i,n;

  int wait=50;
  int counter=1000/wait*len_s;
  while (counter--)
  { frame_delay(wait);
    swap_buffers(1);

    if (actthings<THINGs && (counter&15)>=14)
    { n=firstfree; // if (n<0) ...bad
        if (n<0) { Serial.println(999999); return; }
      actthings++;
      struct thing *e=&things[n];
      firstfree=e->nextfree;
      e->nextfree=-2;
      
      e->x=random(0,SCRdx_fixp-1);
      e->y=0;
      e->dx=random(-20,20+1);
      e->dy=8;
      e->r=random(0,0x14); 
      e->g=random(0,0x14);
      e->b=0x14 -e->r/2 -e->g/2;
      e->r*=e->r;
      e->g*=e->g;
      e->b*=e->b;
      
      e->t=0;
    }
  
    for (int i=0; i<THINGs; i++)
    { struct thing *e=&things[i];
      if (e->nextfree>=-1) continue;
      
      e->x+=e->dx;
      e->y+=e->dy;
      if (0 && (counter&0x1f)==0x1f)
      { e->r-=(e->r>>3);
        e->g-=(e->g>>3);
        e->b-=(e->b>>3);
      }
      e->dy+=(e->t>=0x100 ? FIXP/0xc : FIXP/0x40);
      e->t++;
      if (e->y>SCRdy_fixp-FIXP && e->t<0x100)
      { for (int ii=0; ii<12 && actthings<THINGs; ii++)
        { n=firstfree; // if (n<0) ...bad
            if (n<0) { Serial.println(999999); return; }
          actthings++;
          struct thing *ee=&things[n];
          firstfree=ee->nextfree;
          ee->nextfree=-2;
          ee->x=e->x;
          ee->y=e->y;
          ee->dx=random(-FIXP/4,FIXP/4+1);
          ee->dy=random(-FIXP/1,-FIXP/8);
          ee->r=e->r; 
          ee->g=e->g;
          ee->b=e->b;
          ee->t=0x100;
        }
        e->nextfree=firstfree;
        firstfree=i;
        actthings--;
        break;
      }
      if (e->t>=0x100)
        paint_hi(e->x,e->y,e->r,e->g,e->b);
      else
        for (int ii=0; ii<8; ii++)
          paint_hi(e->x, e->y-ii*(FIXP-3), e->r-ii*0xc, e->g-ii*0xc, e->b-ii*0xc);
      if (e->y>SCRdy_fixp || e->x<0 || e->x>SCRdx_fixp || e->t>=0x10c)
      { e->nextfree=firstfree;
        firstfree=i;
        actthings--;
      }
    }
  }
}

int Ballgame(int len_s)
{ 
  int x=FIXP*2+FIXP/4;
  int y=FIXP*2+FIXP*0;
  int dx=12*4;
  int dy=12*4;
  int r=255+64;
  int g=0*255;
  int b=0*255;
  int pad_x=0;
  int pad_x_old=0;
  int pad_y=(SCRdy-1)*FIXP;
  int pad_w=3;

  int stillcount=0;  
  int bloksleft=0;
  const int blk_maxy=5;
  char blocks[SCRdx][blk_maxy];
  for(int i=0; i<SCRdx; i++)
    for(int j=0; j<blk_maxy; j++)
    { //blocks[i][j]= j<2 ?0 :1;
      b=random(2+4+j)>4;
      blocks[i][j]= b;
      if (b) bloksleft++;
    }
  
  int wait=30;
  int counter=1000/wait*len_s;
  while (counter--)
  { frame_delay(wait);
    swap_buffers(1);

    x+=dx;
      if      (x<0)          { x=0;            dx=-dx; }
      else if (x>SCRdx_fixp-FIXP) { x=SCRdx_fixp-FIXP-1; dx=-dx; }
    y+=dy;
      if      (y<0)          { y=0;            dy=-dy; }
      else if (y>SCRdy_fixp-FIXP +FIXP) { y=SCRdy_fixp-FIXP-1; dy=-dy;  return(0);   }
    paint_hi(x,y, r,g,b);
    //paint(SCRdx-(x+FIXP/2)/FIXP-1,SCRdy-(y+FIXP/2)/FIXP-1, 255,0,0);
    
    int pos=analogRead(A1)/4*SCRdx-FIXP;
    pad_x=pos;
    for(int i=0; i<pad_w; i++)
      paint_hi(pad_x+i*FIXP,pad_y, 0,255,0);
      
    if (y>SCRdy_fixp-FIXP -FIXP &&
        dy>0 &&
        x>=pad_x-FIXP/2 && x<=pad_x+(pad_w-1)*FIXP+FIXP/2
       )
    { //y=SCRdy_fixp-2*FIXP-1; dy=-dy;
      dy=-dy;
      dx=dx/2+(x-(pad_x+FIXP))/8;
    }
    
    int xi=(x+FIXP/2-1)/FIXP;
    int yi=(y+FIXP/2-1)/FIXP;
    if (yi<blk_maxy && blocks[xi][yi])
    { blocks[xi][yi]=0;
      //dx=-dx;
      dy=-dy;
      bloksleft--; if (bloksleft<=0) return(1); // win
    }  
    
    if (abs(pad_x_old-pad_x)<FIXP/2) stillcount++; else stillcount=0;
    // if (stillcount>=30*30) return(0); // 30s
    pad_x_old=pad_x;

    for(int i=0; i<SCRdx; i++)
      for(int j=0; j<blk_maxy; j++)
        if (blocks[i][j]) paint(i,j, 128,128,0);
  }
  return(0); // loose
}


char gespenst_pixel[8][9] = {
                              {0, 0, 1, 1, 1, 1, 1, 0, 0},
                              {0, 1, 1, 1, 1, 1, 1, 1, 0},
                              {1, 1, 1, 1, 1, 1, 1, 1, 1},
                              {1, 1, 1, 1, 1, 1, 1, 1, 1},
                              {1, 1, 1, 1, 1, 1, 1, 1, 1},
                              {1, 1, 1, 1, 1, 1, 1, 1, 1},
                              {1, 1, 1, 1, 1, 1, 1, 1, 1},
                              {1, 0, 1, 0, 1, 0, 1, 0, 1}};
                              
char pacman_gefressen_pixel [6][6][6] = {
  {{0,1,0,0,1,0},
   {1,1,1,1,1,1},
   {1,1,1,1,1,1},
   {1,1,1,1,1,1},
   {1,1,1,1,1,1},
   {0,1,1,1,1,0}},
   
  {{0,0,0,0,0,0},
   {1,1,0,0,1,1},
   {1,1,0,0,1,1},
   {1,1,1,1,1,1},
   {1,1,1,1,1,1},
   {0,1,1,1,1,0}},
   
  {{0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {1,1,0,0,1,1},
   {1,1,1,1,1,1},
   {1,1,1,1,1,1},
   {0,1,1,1,1,0}},
   
  {{0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {0,1,1,1,1,0},
   {1,1,1,1,1,1},
   {0,1,1,1,1,0}},
   
  {{0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {0,0,1,1,0,0},
   {0,1,1,1,1,0},
   {0,1,1,1,1,0}},
   
  {{0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {0,0,0,0,0,0},
   {0,0,1,1,0,0},
   {0,0,1,1,0,0},
   {0,0,1,1,0,0}}
};
char pacman_punkte_pos[7] = {7, 5, 3, 2, 1, 3, 3};
void pacman() {
  swap_buffers(1); //bildschirm leeren
  //randomSeed(micros()/123);
  
  
  boolean richtung = true;
  byte gespenst = random(4);

  boolean gespenst_frisst_pacman = false;
  if (random(1,4) > 1) gespenst_frisst_pacman = true;
  int gespenst_pacman_offset = random(8,SCRdx-16);
  
  
  
  byte module = SCRdx/3;
  
  boolean punkte[module];
  
  for (int i=0; i<module; i++) { // punkte einfliegen
    punkte[i] = true;
    for(int j=0; j<7; j++) {
      swap_buffers(2);
      if (j > 0) paint(2+(i*4), pacman_punkte_pos[j-1], 0, 0, 0);
      paint(2+(i*4), pacman_punkte_pos[j], 255, 255, 255);
      frame_delay(30);
    }
    frame_delay(300);
  }
  frame_delay(1500);
  boolean ende = false;
  byte pacman_status = 0; // 0 = maul auf; 1 = halb zu, schließend; 2 = zu, 3 = halb zu, öffnend; >3 = pacman wird gefressen; 
  
  boolean gespenst_status = true; // gespenster"fuß"status
  byte gespenst_r;
  byte gespenst_g;
  byte gespenst_b;
  
  if (gespenst == 0) {
    gespenst_r = 255;
    gespenst_g = 0;
    gespenst_b = 0;
  }else if(gespenst == 1) {
    gespenst_r = 255;
    gespenst_g = 0;
    gespenst_b = 255;
  }else if(gespenst == 2) {
    gespenst_r = 0;
    gespenst_g = 255;
    gespenst_b = 255;
  }else {
    gespenst_r = 255;
    gespenst_g = 128;
    gespenst_b = 0;
  }
  byte gespenst_pos;
  byte pacman_pos;
  
  if (richtung) {
    pacman_pos = 0;
    gespenst_pos = 0;
  }else {
    pacman_pos = SCRdx;
    gespenst_pos = SCRdx;
  }
  int lauf = 0;
  while (!ende) {
    swap_buffers(1);
    
    //punkte malen
    for (int i=0; i < module; i++) {
      byte x_pos;
      if (richtung) {
        x_pos = 2+i*4;
      }else {
        x_pos = SCRdx - (2+i*4);
      }
      if (punkte[i]) paint(x_pos, 3, 255, 255, 255);
    }
    
    if (richtung) {
      
      //pacman malen
      if (pacman_status < 4) {
        paint(pacman_pos - 4, 1, 255, 255, 0);
        paint(pacman_pos - 3, 1, 255, 255, 0);
        paint(pacman_pos - 2, 1, 255, 255, 0);
        paint(pacman_pos - 1, 1, 255, 255, 0);
        
        paint(pacman_pos - 5, 2, 255, 255, 0);
        paint(pacman_pos - 4, 2, 255, 255, 0);
        paint(pacman_pos - 3, 2, 255, 255, 0);
        paint(pacman_pos - 2, 2, 255, 255, 0);
        paint(pacman_pos - 1, 2, 255, 255, 0);
        paint(pacman_pos    , 2, 255, 255, 0);
        
        paint(pacman_pos - 5, 3, 255, 255, 0);
        paint(pacman_pos - 4, 3, 255, 255, 0);
        paint(pacman_pos - 3, 3, 255, 255, 0);
        paint(pacman_pos - 2, 3, 255, 255, 0);
        
        paint(pacman_pos - 5, 4, 255, 255, 0);
        paint(pacman_pos - 4, 4, 255, 255, 0);
        paint(pacman_pos - 3, 4, 255, 255, 0);
      
        if (pacman_status == 0) {
          pacman_status = 1;
        }else if (pacman_status == 1) {
          paint(pacman_pos - 1, 3, 255, 255, 0);
          paint(pacman_pos    , 3, 255, 255, 0);
          pacman_status = 2;
        }else if (pacman_status == 2) {
          paint(pacman_pos - 2, 3, 255, 255, 0);
          paint(pacman_pos - 1, 3, 255, 255, 0);
          paint(pacman_pos    , 3, 255, 255, 0);
          paint(pacman_pos - 2, 4, 255, 255, 0);
          paint(pacman_pos - 1, 4, 255, 255, 0);
          paint(pacman_pos    , 4, 255, 255, 0);
          pacman_status = 3;
        }else if (pacman_status == 3) {
          paint(pacman_pos - 1, 3, 255, 255, 0);
          paint(pacman_pos    , 3, 255, 255, 0);
          pacman_status = 0;
        }
        
        
        paint(pacman_pos - 5, 5, 255, 255, 0);
        paint(pacman_pos - 4, 5, 255, 255, 0);
        paint(pacman_pos - 3, 5, 255, 255, 0);
        paint(pacman_pos - 2, 5, 255, 255, 0);
        paint(pacman_pos - 1, 5, 255, 255, 0);
        paint(pacman_pos    , 5, 255, 255, 0);
        
        paint(pacman_pos - 4, 6, 255, 255, 0);
        paint(pacman_pos - 3, 6, 255, 255, 0);
        paint(pacman_pos - 2, 6, 255, 255, 0);
        paint(pacman_pos - 1, 6, 255, 255, 0);
        
        
        if ((pacman_pos+1) % 4 == 0) {
         if (((pacman_pos+1)/4)-1>=0 && ((pacman_pos+1)/4)-1<module) // security check
          punkte[((pacman_pos+1)/4)-1] = false;
        }
      
        //gespenst malen

        if (pacman_pos > gespenst_pacman_offset) {
          for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 9; j++) {
              if (gespenst_pixel[i][j]) paint(gespenst_pos-j, i, gespenst_r, gespenst_g, gespenst_b);
            }
          }
          
          //augen malen
          paint(gespenst_pos - 2, 2, 255, 255, 255);
          paint(gespenst_pos - 3, 3, 255, 255, 255);
          paint(gespenst_pos - 3, 2, 255, 255, 255);
          paint(gespenst_pos - 2, 3, 0,   0,   0);
          
          paint(gespenst_pos - 5, 2, 255, 255, 255);
          paint(gespenst_pos - 6, 3, 255, 255, 255);
          paint(gespenst_pos - 6, 2, 255, 255, 255);
          paint(gespenst_pos - 5, 3, 0,   0,   0);
          
          if (!gespenst_status) {
             for (int f = 0; f < 9; f++) {
               if (f % 2 == 0) {
                 paint(gespenst_pos-f, 7, 0, 0, 0);
               }else {
                 paint(gespenst_pos-f, 7, gespenst_r, gespenst_g, gespenst_b);
               }
             }
          }
          gespenst_status = !gespenst_status;
        }
      
      }
      
      if ((gespenst_pos != 0) && ((pacman_pos - gespenst_pos) < 4)) {
        if (pacman_status < 3) pacman_status = 3;
        if (pacman_status == 3) frame_delay(222);
        pacman_status++;
        
        for (int n = 0; n < 6; n++) {
         for (int m = 0; m < 6; m++) {
           if ((pacman_status < 9) && pacman_gefressen_pixel[pacman_status-4][m][n]) paint(pacman_pos - 5 + n, m, 255, 255, 0);
         } 
        }
        frame_delay(50);
      }
      
      if (pacman_status < 4) {
        if (lauf % 2 == 0) pacman_pos++;
        if ((lauf % 2 == 0) && (pacman_pos > gespenst_pacman_offset)) {
          if (gespenst_frisst_pacman) {
            gespenst_pos += 2;
          } else {
            gespenst_pos++;
          }
        }
      }
      lauf++;
      
      if ((gespenst_pos - 16) > SCRdx) ende = true;
      if (pacman_status > 20) ende = true;
      if (lauf == 100) ende = true;
      
      frame_delay(100);
    }else {
      
    }
  }
}

int BallgameTtip(int len_s)
{ 
  int x=FIXP*2+FIXP/4;
  int y=FIXP*2+FIXP*0;
  int dx=12*4;
  int dy=6*4;
  int r=255+64;
  int g=0*255;
  int b=0*255;
  int pad_x=0;
  int pad_x_old=0;
  int pad_y=(SCRdy-1)*FIXP;
  int pad_w=3;

  int stillcount=0;  
  int bloksleft=0;
  const int blk_maxx=18;
  const int blk_maxy=7;
  char blocks[blk_maxx][blk_maxy];
  for(int i=0; i<SCRdx; i++)
    for(int j=0; j<blk_maxy; j++)
    { //blocks[i][j]= j<2 ?0 :1;
      //b=random(2+4+j)>4;
      b=0;
      blocks[i][j]= b;
      if (b) bloksleft++;
    }
  b=1;  
    
  //blocks[1][1]= b;
  blocks[2][1]= b;
  blocks[3][1]= b;
  blocks[4][1]= b;
  //blocks[5][1]= b;
  blocks[3][2]= b;
  blocks[3][3]= b;
  blocks[3][4]= b;
  blocks[3][5]= b;
  blocks[3][6]= b;

  blocks[6][1]= b;
  blocks[7][1]= b;
  blocks[8][1]= b;
  blocks[7][2]= b;
  blocks[7][3]= b;
  blocks[7][4]= b;
  blocks[7][5]= b;
  blocks[7][6]= b;
  
  blocks[10][1]= b;
  blocks[10][2]= b;
  blocks[10][3]= b;
  blocks[10][4]= b;
  blocks[10][5]= b;
  blocks[10][6]= b;

  blocks[13][1]= b;
  blocks[14][1]= b;
  blocks[15][1]= b;
  blocks[13][2]= b;
  blocks[13][3]= b;
  blocks[13][4]= b;
  blocks[13][5]= b;
  blocks[13][6]= b;
  blocks[14][4]= b;
  blocks[15][4]= b;
  blocks[16][2]= b;
  blocks[16][3]= b;

  bloksleft=0;
  for(int i=0; i<SCRdx; i++)
    for(int j=0; j<blk_maxy; j++)
      if (blocks[i][j]) bloksleft++;

  int wait=10;
  int counter=1000/wait*len_s;
  while (counter--)
  { frame_delay(wait);
    swap_buffers(1);

    int rr=4;
    x+=random(-rr,rr+1);
    y+=random(-rr,rr+1);

    x+=dx;
      if      (x<0)          { x=0;            dx=-dx; }
      else if (x>SCRdx_fixp-FIXP) { x=SCRdx_fixp-FIXP-1; dx=-dx; }
    y+=dy;
      if      (y<0)          { y=0;            dy=-dy; }
      else if (y>SCRdy_fixp-FIXP +FIXP) { y=SCRdy_fixp-FIXP-1; dy=-dy;  }
    paint_hi(x,y, r,g,b);
    
    int xi=(x+FIXP/2-1)/FIXP;
    int yi=(y+FIXP/2-1)/FIXP;
    if (yi<blk_maxy && blocks[xi][yi])
    { blocks[xi][yi]=0;
      //dx=-dx;
      dy=-dy;
      bloksleft--; if (bloksleft<=0) return(1); // win
    }  

    for(int i=0; i<SCRdx; i++)
      for(int j=0; j<blk_maxy; j++)
        if (blocks[i][j]) paint(i,j, 128,128,0);
  }
  return(0); // loose
}

int appleman_xy(float cx, float cy)
{ int maxiter=64;
  float x=0, y=0, xx=0, d=0;
  int i;
  for (i=0; i<maxiter && d<=4; i++)
  { // z = z²+c
    d =x*x+y*y;
    xx=x*x-y*y+cx;
    y = 2*x*y +cy;
    x =xx;
  }
  return(i);
}
void appleman(float x0, float x1, float y0, float y1)
{ float cx,cy;
  float dx=(x1-x0)/SCRdx;
  float dy=(y1-y0)/SCRdy;
  int x,y,ox,oy,depth;
  
  const int oxmax=2,oymax=2;
  float ocx,ocy;
  float odx=dx/oxmax;
  float ody=dy/oymax;
  
  int r,g,b;
  for (x=0, cx=x0; x<SCRdx; x++, cx+=dx)
    for (y=0, cy=y0; y<SCRdy; y++, cy+=dy)
    { r=0; g=0, b=0;
      for (ox=0, ocx=cx; ox<oxmax; ox++, ocx+=odx)
        for (oy=0, ocy=cy; oy<oymax; oy++, ocy+=ody)
        { depth=appleman_xy(ocx,ocy);
          if ((depth>>3)&1) r+=1;
          if ((depth>>3)&2) g+=1;
          if ((depth>>3)&4) b+=1;
        }
      paint(x,y, r*256/(oxmax*oymax),
                    g*256/(oxmax*oymax),
                    b*256/(oxmax*oymax));
    }
}
void Appleman(int len_s)
{ //float cx=0.02;
  //float cy=0.848;
  //float cx=-0.65916820654497, cy=-0.45032886965735;
  float cx=-1.0278707411048, cy=-0.36128217904636;
  float dx=8;
  float dy=(dx*SCRdy)/SCRdx;
  //float x0,x1,y0,y1;
  
  int wait=100;
  int counter=1000/wait*len_s;
  while (counter-- && dx>0.001)
  { frame_delay(wait);
    swap_buffers(1);
    
    appleman(cx-dx,cx+dx,cy-dy,cy+dy);
    dx*=0.8;
    dy=(dx*SCRdy)/SCRdx;
  }
}
void Testmuster(int len_s)
{ int shift=0;
  int st=0;
  int en=8;

  int wait=1000;
  int counter=1000/wait*len_s;
  while (counter>0)
  { /*for (int i=st; i<en; i++)      paint(shift,i,255,0,0);
    swap_buffers(1);
    frame_delay(wait); counter--;
  
    for (int i=st; i<en; i++)      paint(shift,i,0,255,0);
    swap_buffers(1);
    frame_delay(wait/2); //counter--;
    */
    for (int i=st; i<en; i++)      paint(shift,i,0,0,255);
    swap_buffers(1);
    frame_delay(wait/2); counter--;
    
    shift++; if (shift>=SCRdx) shift=0;
  }
}

void Plasma1(int len_s)
{ const int POINTs = 3;
  struct point
  { int x,y;
    int r,g,b;
    int d;
  } points[POINTs];
  for (int i=0; i<POINTs; i++)
  { points[i].x=random(0,FIXP*SCRdx);
    points[i].y=random(0,FIXP*SCRdy);
    points[i].r=random(0,255);
    points[i].g=random(0,255);
    points[i].b=random(0,255);
    points[i].d=0;
  }
  
  int wait=100;                  // std loop (defines 100 ms frames)
  int counter=1000/wait*len_s;   // std loop (takes parameter len_s length in seconds)
  while (counter--)              // std loop
  { frame_delay(wait);           // std loop (waits till last frame_delay endtime plus wait ms)
    swap_buffers(1);             // std loop (double buffering, as expected with starts blank=black)
    
    int dd=16;
    for (int i=0; i<POINTs; i++)
    { points[i].x+=random(-FIXP/2,FIXP/2+1);
        if (points[i].x<0) points[i].x=0; else if (points[i].x>=FIXP*SCRdx) points[i].x=FIXP*SCRdx-1;
      points[i].y+=random(-FIXP/2,FIXP/2+1);
        if (points[i].y<0) points[i].y=0; else if (points[i].y>=FIXP*SCRdy) points[i].y=FIXP*SCRdy-1;
      points[i].r+=random(-dd,dd+1);
        if (points[i].r<0) points[i].r=0; else if (points[i].r>255) points[i].r=255;
      points[i].g+=random(-dd,dd+1);
        if (points[i].g<0) points[i].g=0; else if (points[i].g>255) points[i].g=255;
      points[i].b+=random(-dd,dd+1);
        if (points[i].b<0) points[i].b=0; else if (points[i].b>255) points[i].b=255;
    }
    for (int x=0; x<SCRdx; x++)
      for (int y=0; y<SCRdx; y++)
      { int dsum=0;
        int dmax=0;
        for (int i=0; i<POINTs; i++)
        { //int d= abs(points[i].x-x*FIXP) +abs(points[i].y-y*FIXP);
          int d= sqrt((points[i].x-x*(long)FIXP)*(points[i].x-x*(long)FIXP) +(points[i].y-y*(long)FIXP)*(points[i].y-y*(long)FIXP));
          //d=d*points[i].r/255;
          if (d>0) d=SCRdx*FIXP/d;
          else     d=SCRdx*FIXP;
          //d*=d/16+16*d;
          points[i].d=d;
          if (d>dmax) dmax=d;
          dsum+=d;
        } 
        int br=0;
        int r=0;
        int g=0;
        int b=0;
        for (int i=0; i<POINTs; i++)
        { //b+=points[i].d*255/dsum;
          r+=points[i].d*255/dmax*points[i].r/255;
          g+=points[i].d*255/dmax*points[i].g/255;
          b+=points[i].d*255/dmax*points[i].b/255;
          br+=points[i].d*255/dmax;
          //if (b>0) b=255/b;
        } 
        br/=POINTs;
        //paint(x,y,br,br/2,br/4);
        //paint(x,y,r/POINTs*2,g/POINTs*2,b/POINTs*2);
        
        paint(x,y, r/POINTs/2+br/POINTs*2, g/POINTs/2+br/POINTs*2, b/POINTs/2+br/POINTs*2);
        //paint(x,y, r/POINTs/2+br/POINTs, g/POINTs/2+br/POINTs, b/POINTs/2+br/POINTs);
        //paint(x,y, r/POINTs+br/POINTs, g/POINTs+br/POINTs, b/POINTs+br/POINTs);
        //paint(x,y, r/2+br/POINTs, g/2+br/POINTs, b/2+br/POINTs);
        //paint(x,y, r/POINTs+br, g/POINTs+br/1, b/POINTs+br/1);
        //paint(x,y, r/POINTs, g/POINTs, b/POINTs);
        
        //paint(x,y,br*r/255, br*g/255, br*b/255);
      }
  }                              // std loop end
}

void loop()                     
{ // debug method: can be used inside animations...
  // digitalWrite(pinDebug, act_buf_write ? HIGH : LOW );

  if(0) // primarily used for triangular matrix
  { Testmuster(24);
    pacman();
    return;
  }
  if (0)
  { Sparkles(20);
    //Plasma1(20);
    return;
  }

  string_anim("NEW GAME",400,1, 255,0,0);
  if (!Ballgame(10))
  { string_anim("LOST",400,1, 128,0,255);
    SlimeDrops(8);
  }
  else
  { string_anim("SUCCESS",400,1, 255,128,0);
    Fireworks(12);
  }
  Plasma1(3);
  Sparkles(8);
  
  string_anim("KIEL",400,1, 0,127,255);
  pacman();

  string_anim("TOPPOINT",400,1, 255,127,0);
  Fireworks(6);

  string_anim("MINI MATRIX",400,1, 255,255,0);
  KnightRider();

  string_anim("FREIFUNK",400,1, 64,255,64);
  Appleman(16);

  pacman();

  BallgameTtip(6);

  string_anim("FORESTFIRE",400,1, 64,255,64);
  ForestFire(10);
  
  /*string_anim("FRICKL",400,1, 255,128,0);
  string_anim("  FRICKL  ",400,1, 255,64,0);
  string_anim("  MINI MATRIX  ",400,1, 255,255,0);
  string_anim("  TOPPOINT  ",400,1, 255,127,0);
  string_anim("  KIEL  ",400,1, 0,127,255);
  string_anim("  FREIFUNK  ",400,1, 64,255,64);
  */
}

